/*
 *  CopyRight (c) 2008 by OpenMoko, Inc.  
 *  Matt Hsu <matt_hsu@openmoko.org> 
 *  tiny client to obtain A-GPS data 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 *  See the *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdlib.h>
#include <stdio.h>
#include <termios.h> 
#include <string.h>

#include "agps-onlinec.h"
#include "ubx.h"

#define CONTENT_LENGTH 	"Content-Length: "
#define CONTENT_TYPE 	"Content-Type: "
#define UBX_FORMAT 	"application/ubx"

#define BUFFER_LEN 	2048

/* AGPS server information */
static const char *ubx_agps_ser = "agps.u-blox.com";
static const int  ser_port = 46434;

static char agps_buf[4096];
struct agps_onlinec_data agps;

static const char *key[] = {
	"cmd=", "user=", "pwd=",
	"lat=", "lon=", "alt=", "pacc=",
	"latency=",
};

static void usage(void)
{
	puts("Usage: ");
	puts("-c, --command 	request to get what kind of information from server");
	puts("-u, --user 	username");
	puts("-k, --passwd  	password");
	puts("-h, --help  	show all the arguments");
	puts("-la, --latitude  	approximate user position in Latitude");
	puts("-lo, --longitude 	approximate user position in Longitude");
	puts("-al, --altitude 	approximate user altitude");
	puts("-p, --pacc 	approximate accuracy of submitted position");
	puts("-d, --latency 	typical latency between the time the server receives the request");
	puts("example: agps-onlinec -c full -u foo -k passwd -la 25.073270 -lo 121.574805 -al 46.7 -p 999999.00 ");
}

int establish_conn(void)
{	
	struct sockaddr_in remote;
	struct hostent *host;
	char buf[BUFFER_LEN];
	int ret, n, i;
	unsigned char *tmp = NULL;

	/* allocating socket */
	memset(&remote, 0, sizeof(remote));
	remote.sin_family = AF_INET;	
	agps.sd = socket(PF_INET, SOCK_STREAM, 0);
	remote.sin_port =  htons(ser_port);
	
	/* reslove host name to IP addr */
	host = gethostbyname(ubx_agps_ser);

	if (host != 0)
		memcpy(&remote.sin_addr, host->h_addr, host->h_length);
	else{
		printf("can't reslove host name \n");
		return 1;
	}

	connect(agps.sd, (struct sockaddr *)&remote, sizeof(remote));
	
	memset(buf, 0, BUFFER_LEN);
	tmp = buf;

	/* send request */
	for (i = CMD; i < MAX_ITEM; i++){
		if (agps.request[i] != NULL){
			strcat(tmp, key[i]);
			strcat(tmp, agps.request[i]);
			strcat(tmp, ";");
		}
	}
	strcat(tmp, "\n");
	
	write(agps.sd, tmp, strlen(tmp));	
	
	/* get response */
	memset(buf, 0, BUFFER_LEN);
	tmp = agps_buf;
	
	while ( (n = read(agps.sd, buf, BUFFER_LEN)) > 0){
		memcpy(tmp, buf, n);
		tmp = tmp + n;
	}

	return 0;

}

int main (int argc, char *argv[])
{	
	int i, n, aid_length;	
	char 	**opt_arg;
	unsigned char *tmp = NULL;

	opt_arg = argv + 1;

	if (!*opt_arg){
		usage();
		return 1;
	}

	/* parse arguments */
	while (*opt_arg){
		if (strcmp(*opt_arg, "-c") == 0 || strcmp(*opt_arg, "--command") == 0){
			if (!*(++opt_arg)){
				usage();
				return 1;
			}
			agps.request[CMD] = *opt_arg;
		}
		else if(strcmp(*opt_arg, "-h") == 0 || strcmp(*opt_arg, "--help") == 0 ){
				usage();
				return 1;
		}
		else if(strcmp(*opt_arg, "-u") == 0 || strcmp(*opt_arg, "--user") == 0 ){
			if (!*(++opt_arg)){
				usage();
				return 1;
			}
			agps.request[USR] = *opt_arg;
		}
		else if(strcmp(*opt_arg, "-k") == 0 || strcmp(*opt_arg, "--passwd") == 0 ){
			if (!*(++opt_arg)){
				usage();
				return 1;
			}
			agps.request[PASSWD] = *opt_arg;
		}
		else if(strcmp(*opt_arg, "-la") == 0 || strcmp(*opt_arg, "--latitude") == 0 ){
			if (!*(++opt_arg)){
				usage();
				return 1;
			}	
			agps.request[LAT] = *opt_arg;
		}
		else if(strcmp(*opt_arg, "-lo") == 0 || strcmp(*opt_arg, "--longitude") == 0 ){
			if (!*(++opt_arg)){
				usage();
				return 1;
			}
			agps.request[LON] = *opt_arg;
		}
		else if(strcmp(*opt_arg, "-al") == 0 || strcmp(*opt_arg, "--altitude") == 0 ){
			if (!*(++opt_arg)){
				usage();
				return 1;
			}
			agps.request[ALT] = *opt_arg;
		}
		else if(strcmp(*opt_arg, "-p") == 0 || strcmp(*opt_arg, "--pacc") == 0 ){
			if (!*(++opt_arg)){
				usage();
				return 1;
			}
			agps.request[PACC] = *opt_arg;
		}
		else if(strcmp(*opt_arg, "-d") == 0 || strcmp(*opt_arg, "--latency") == 0 ){
			if (!*(++opt_arg)){	
				usage();
				return 1;
			}
			agps.request[LATENCY] = *opt_arg;
		}
		++opt_arg;
	}

	if (establish_conn())
		goto parsing_fail;
	else
		printf("connect to %s \n", ubx_agps_ser);

	/* content length */
	tmp = strstr(agps_buf, CONTENT_LENGTH);

	if (!tmp){
		printf("content_length is missing, msg might be broken \n");
		goto parsing_fail;
	}

	tmp = tmp + strlen(CONTENT_LENGTH);
	printf("aid data %d bytes received\n", atoi(tmp));

	aid_length = atoi(tmp);
	
	/* content type */
	if (tmp = strstr(agps_buf, CONTENT_TYPE)){
		if (tmp = strstr(agps_buf, UBX_FORMAT))
			/* skip 2 empty line */
			tmp = tmp + strlen(UBX_FORMAT) + 4;
	}
	else{
		printf("content_type is missing, msg might be broken \n");
		goto parsing_fail;	
	}	
		
	/* open device file */
	dev_open();

	/* resetting GPS */
	gps_reset();

	if (ubx_raw_write(tmp, aid_length))		
		printf("aid data raw written failed\n");
	else
		printf("aid data raw written ok\n");

	close(agps.sd);	
	close(agps.fd);

	return 0;

parsing_fail:
	close(agps.sd);
	return -1;
}
